package fr.smallcrew.utils;

import com.google.common.base.Optional;
import com.google.common.base.Splitter;
import com.google.common.collect.Iterables;
import org.apache.commons.mail.DefaultAuthenticator;
import org.apache.commons.mail.Email;
import org.apache.commons.mail.SimpleEmail;
import org.springframework.util.StringUtils;

import javax.mail.Authenticator;
import java.util.Locale;
import java.util.Map;

import static com.google.common.base.Throwables.propagate;
import static com.google.common.collect.Maps.newHashMap;
import static fr.smallcrew.utils.TemplateHelper.template;

public class Mailer {
  private final Optional<Authenticator> authenticator;
  private final String hostName;
  private final String from;
  private final int port;
  private final boolean useTLS;

  public Mailer(String hostName, int port, Authenticator authenticator, boolean useTLS, String from) {
    this.hostName = hostName;
    this.port = port;
    this.useTLS = useTLS;
    this.from = from;
    this.authenticator = Optional.fromNullable(authenticator);
  }

  public Mailer(String hostName, int port, String login, String password, boolean useTLS, String from) {
    this(hostName, port, newDefaultAuthenticator(login, password), useTLS, from);
  }

  private static Authenticator newDefaultAuthenticator(String login, String password) {
    if (StringUtils.isEmpty(login) && StringUtils.isEmpty(password)) {
      return null;
    }
    return new DefaultAuthenticator(login, password);
  }

  public Mailer(String hostName, int port, String from) {
    this(hostName, port, null, false, from);
  }

  public Sender mailTo(String to) {
    return new Sender(to);
  }

  public class Sender {
    private final Map<String, Object> data;
    private final String to;
    private String templateName;
    private Locale locale;

    public Sender(String to) {
      this.data = newHashMap();
      this.to = to;
    }

    public Sender withMsgTemplate(String template) {
      this.templateName = template;
      return this;
    }

    public Sender put(String varName, Object value) {
      this.data.put(varName, value);
      return this;
    }

    public Sender putAll(Map<String, Object> data) {
      this.data.putAll(data);
      return this;
    }

    public Sender withLocale(Locale locale) {
      this.locale = locale;
      return this;
    }

    public void send() {
      try {
        String msg = template(templateName).withLocale(locale).putAll(data).asString();
        Iterable<String> subjectAndText = Splitter.on("===").split(msg);
        Email email = new SimpleEmail()
          .addTo(to)
          .setFrom(from)
          .setSubject(Iterables.get(subjectAndText, 0, "").trim())
          .setMsg(Iterables.get(subjectAndText, 1, "").trim());
        email.setHostName(hostName);
        email.setSmtpPort(port);
        if (authenticator.isPresent()) {
          email.setStartTLSEnabled(useTLS);
          email.setAuthenticator(authenticator.get());
        }
        email.send();
      } catch (Exception e) {
        throw propagate(e);
      }
    }
  }
}
